//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;
#import <Diffusion/PTDiffusionFieldMetadata.h>

NS_ASSUME_NONNULL_BEGIN

/**
 @brief Number field metadata describes a data item that can have a numerical value.

 A number field metadata object is immutable.

 Default multiplicity is defined by the `defaultMultiplicity` class method on PTDiffusionMultiplicity.

 @see PTDiffusionRecordMetadata

 @since 5.6
 */
@interface PTDiffusionNumberFieldMetadata : PTDiffusionFieldMetadata

/**
 The default scale. It has a value of zero (`0`), which constrains values to integer.
 
 @since 5.6
 */
+(NSUInteger)defaultScale;

/**
 The default value used if one is not explicitly specified on initialisation.
 
 @return A decimal number object equivalent to the number 0.0.
 
 @since 5.7.1
 */
+(NSDecimalNumber *)defaultDefaultValue;

/**
 Returns a number field metadata object initialized with the given name, given scale
 and given multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @param scale The scale to use. It governs how many digits lie to the right
 of the decimal point. A value of zero (`0`) constrains transformed fields to integer values.

 @exception NSInvalidArgumentException If either the given name or multiplicity is `nil`.

 @since 5.6
 */
-(instancetype)initWithName:(NSString *)name
               multiplicity:(PTDiffusionMultiplicity *)multiplicity
                      scale:(NSUInteger)scale;

/**
 Returns a number field metadata object initialized with the given name, given default
 value, given scale and given multiplicity.

 @param name The name to use. It is used when looking up fields that have been
 transformed using the receiver.
 
 @param defaultValue The default value that required occurrences of the field should
 be initialized to. Specifying `nil` for this argument uses a value equivalent
 to the number 0.0 (as specified by the defaultDefaultValue class method).

 @param multiplicity The multiplicity to use. It governs how many fields
 can be transformed using the receiver from the enclosing record.

 @param scale The scale to use. It governs how many digits lie to the right
 of the decimal point. A value of zero (`0`) constrains transformed fields to integer values.

 @exception NSInvalidArgumentException If either the given name or multiplicity is `nil`.

 @since 5.7.1
 */
-(instancetype)initWithName:(NSString *)name
               defaultValue:(nullable NSNumber *)defaultValue
               multiplicity:(PTDiffusionMultiplicity *)multiplicity
                      scale:(NSUInteger)scale NS_DESIGNATED_INITIALIZER;

/**
 The default value that required occurrences of the field should be initialized to.
 
 @since 5.7.1
 */
@property(nonatomic, readonly) NSNumber* defaultValue;

/**
 The scale for fields transformed using the receiver, being the number of digits
 to the right of the decimal point.
 
 A value of zero (`0`) constrains fields to integer values.

 @since 5.6
 */
@property(nonatomic, readonly) NSUInteger scale;

/**
 Compares the receiver to the given number field metadata.

 @param numberFieldMetadata The number field metadata object with which to compare the receiver.

 @return `YES` if the number field metadata is equal to the receiver, otherwise `NO`.

 @since 5.6
 */
-(BOOL)isEqualToNumberFieldMetadata:(nullable PTDiffusionNumberFieldMetadata *)numberFieldMetadata;

@end

NS_ASSUME_NONNULL_END
